import 'dart:math';

import 'package:calendar_view/utils/calendar_util.dart';
import 'package:flutter/material.dart';

class ScheduleList extends StatefulWidget {
  const ScheduleList({Key? key, required this.maxLines}) : super(key: key);

  /// 可容纳的最大日程行数
  final int maxLines;

  /// 每行日程的高度
  final double itemHeight = 20;

  @override
  ScheduleListState createState() => ScheduleListState();
}

class ScheduleListState extends State<ScheduleList> {
  static final List<ScheduleType> _mockScheduleTypeList = [
    ScheduleType(typeName: '工作', isSelected: true, color: Colors.blue),
    ScheduleType(typeName: '加班', isSelected: false, color: Colors.red),
    ScheduleType(typeName: '学习', isSelected: false, color: Colors.green),
    ScheduleType(typeName: '生活', isSelected: false, color: Colors.orange),
    ScheduleType(typeName: '核酸', isSelected: false, color: Colors.purple),
    ScheduleType(typeName: '工作', isSelected: true, color: Colors.blue),
    ScheduleType(typeName: '加班', isSelected: false, color: Colors.red),
    ScheduleType(typeName: '学习', isSelected: false, color: Colors.green),
    ScheduleType(typeName: '生活', isSelected: false, color: Colors.orange),
    ScheduleType(typeName: '核酸', isSelected: false, color: Colors.purple),
  ];
  List<ScheduleType> _scheduleTypeList = [];

  /// 容纳的数量
  int _itemCount = 0;

  @override
  void initState() {
    super.initState();

    /// 填充假数据
    final int random = Random().nextInt(10);
    _scheduleTypeList = _mockScheduleTypeList.sublist(0, random);
    _itemCount = _scheduleTypeList.length > widget.maxLines
        ? widget.maxLines
        : _scheduleTypeList.length;
  }

  @override
  void didUpdateWidget(ScheduleList oldWidget) {
    super.didUpdateWidget(oldWidget);
    _itemCount = _scheduleTypeList.length > widget.maxLines
        ? widget.maxLines
        : _scheduleTypeList.length;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(horizontal: 2),
      child: _buildScheduleTypeList(),
    );
  }

  Widget _buildScheduleTypeList() {
    return ListView.builder(
      addAutomaticKeepAlives: false,
      addRepaintBoundaries: true,
      itemExtent: widget.itemHeight,
      itemCount: _itemCount,
      itemBuilder: (context, index) {
        return _buildScheduleTypeItem(index);
      },
    );
  }

  Widget _buildScheduleTypeItem(int scheduleIndex) {
    /// TODO: 判断如果构建的是展示的最后一项，但不是列表的最后一项，则需要添加点击事件，并显示“还有${_scheduleTypeList.length - _itemCount + 1}项...”
    if (scheduleIndex == _itemCount - 1) {
      if (_itemCount < _scheduleTypeList.length) {
        return _buildMoreItemButton(_scheduleTypeList.length - _itemCount + 1);
      }
    }
    ScheduleType scheduleType = _scheduleTypeList[scheduleIndex];
    return Material(
      color: Colors.transparent,
      child: InkWell(
        customBorder: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(5),
        ),
        onTap: () => print('点击了${scheduleType.typeName}'),
        hoverColor: const Color.fromARGB(255, 227, 228, 228),
        child: Container(
          height: widget.itemHeight,
          padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 0),
          decoration: BoxDecoration(
            color: scheduleType.isSelected
                ? scheduleType.color.withOpacity(0.2)
                : Colors.transparent,
            borderRadius: BorderRadius.circular(5),
          ),
          child: Row(
            children: [
              Container(
                width: 6,
                height: 6,
                decoration: BoxDecoration(
                  color: scheduleType.color,
                  borderRadius: BorderRadius.circular(5),
                ),
              ),
              const SizedBox(width: 10),
              Text(
                scheduleType.typeName,
                style: const TextStyle(fontSize: 12),
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _buildMoreItemButton(int moreNum) {
    return Material(
      color: Colors.transparent,
      child: InkWell(
        customBorder: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(5),
        ),
        onTap: () => print('点击了更多'),
        hoverColor: const Color.fromARGB(255, 227, 228, 228),
        child: Container(
          height: widget.itemHeight,
          padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 0),
          color: Colors.transparent,
          child: Row(
            children: [
              Container(
                width: 6,
                height: 6,
                decoration: BoxDecoration(
                  color: Colors.transparent,
                  borderRadius: BorderRadius.circular(5),
                ),
              ),
              const SizedBox(width: 10),
              Text(
                '还有$moreNum项...',
                style: const TextStyle(
                  fontSize: 12,
                  color: Colors.black38,
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
